"""# googleauth



1. 安装 django-google-auth2


```
pip3 install django-google-auth2
```

2. 添加 django_google_auth2 到app

```
INSTALLED_APPS = [
        ...
        'django_google_auth2',
    ]
```

3. 绑定google令牌

```
from django_google_auth2.google.bindgoogleauth.bindgoogleauth import bind_google_auth

bind_google_auth(user)

```


**函数参数：**

|参数名|必选|类型|说明|
|:----    |:---|:----- |-----   |
|user |是  |string | 用户名 Or 邮箱   |


 **返回参数说明**
|参数名|类型|说明|
|:-----  |:-----|-----                           |
|success |bool   | True/False |
|data |string   | google令牌字符串(用于生成二维码) |

4. 解绑google令牌

```
from django_google_auth2.google.deletegoogleauth.deletegoogleauth import delete_google_auth


delete_google_auth(user)

```


**函数参数：**

|参数名|必选|类型|说明|
|:----    |:---|:----- |-----   |
|user |是  |string | 用户名 Or 邮箱   |

 **返回参数说明**
|参数名|类型|说明|
|:-----  |:-----|-----|
|success |bool   | True/False |
|data |string   | 删除成功 |

5. 验证google令牌

```
from django_google_auth2.google.checkgoogleauth.checkgoogleauth import check_google_auth



check_google_auth(user,code)

```

**函数参数：**

|参数名|必选|类型|说明|
|:----    |:---|:----- |-----   |
|user |是  |string | 用户名 Or 邮箱   |
|code |是  |string | 客户端动态码    |

 **返回参数说明**
|参数名|类型|说明|
|:-----  |:-----|-----|
|success |bool   | True/False |


6. 绑定google令牌Api接口


urls.py

```
from django_google_auth2.google.bindgoogleauth.bindgoogleauthapi import bind_google_auth_api


urlpatterns = [
    ...
    path('bing-google-auth-api/', bind_google_auth_api),
]

```

**请求方式：**
- POST

 **请求示例**

```
{
	"user":"cox"
}
```

**函数参数：**

|参数名|必选|类型|说明|
|:----    |:---|:----- |-----   |
|user |是  |string | 用户名 Or 邮箱   |

 **返回**

![20190402192923-image.png](https://raw.githubusercontent.com/Coxhuang/yosoro/master/20190402192923-image.png)


7. 客户端(二选一)

> 安卓App

Google令牌+扫码器(如果手机只安装Google令牌App扫码失败,请安装扫码器)

链接：https://pan.baidu.com/s/1XeO7p4IvNuvzQOiZrq4wtw

提取码：e70f

> Chrome插件

https://chrome.google.com/webstore/detail/authenticator/bhghoamapcdpbohphigoooaddinpkbai

"""
from django.contrib import admin
from django.urls import path,include
from django_google_auth2.google.bindgoogleauth.bindgoogleauthapi import bind_google_auth_api


urlpatterns = [
    path('bing-google-auth-api/', bind_google_auth_api),
]



from django_google_auth2.models import DjangoGoogleAuthenticator2
import pyotp






def check_google_auth(user,code):
    """
    验证google令牌
    :param user: 邮箱
    :param code: 客服端动态码
    :return: True/False
    """

    queryset_user = DjangoGoogleAuthenticator2.objects.filter(
        username=user
    )

    if not queryset_user.exists():
        return "{}用户未绑定google令牌".format(user)

    obj_user = queryset_user.first()
    key = obj_user.key # 用户秘钥

    t = pyotp.TOTP(key)
    result = t.verify(code)  # 对输入验证码进行校验，正确返回True
    ret = result if result is True else False

    return {"success":ret}



from django.shortcuts import render,HttpResponse
from django_google_auth2.google.bindgoogleauth.bindgoogleauth import bind_google_auth
import json


def bind_google_auth_api(request):

    body = json.loads(request.body)

    data= bind_google_auth(body["user"])

    status = data["success"]

    if not status:
        return HttpResponse(data["data"])

    return render(request, 'google.html', {"qr_code": data["data"]})


from django_google_auth2.models import DjangoGoogleAuthenticator2



# from django_google_auth2.google.deletegoogleauth.deletegoogleauth import delete_google_auth

def delete_google_auth(user):
    """
    删除google令牌
    :param user: 邮箱
    :return: None
    """

    DjangoGoogleAuthenticator2.objects.filter(
        username=user
    ).delete()



    return  {
        "success":True,
        "data":"删除成功"
    }




from django_google_auth2.models import DjangoGoogleAuthenticator2
import base64,codecs,random,re
from django_google_auth2.utils.googletotp import googletotp





# from django_google_auth2.google.bindgoogleauth.bindgoogleauth import bind_google_auth

def bind_google_auth(user):
    """
    绑定google令牌
    :param user:  用户邮箱
    :return: google令牌二维码
    """

    queryset_user = DjangoGoogleAuthenticator2.objects.filter(
        username=user
    )

    if queryset_user.exists():
        return {
            "success":False,
            "data":"{}用户已经绑定google令牌,不能二次绑定".format(user)
        }

    base_32_secret = \
        base64.b32encode(
            codecs.decode(codecs.encode(
                '{0:020x}'.format(random.getrandbits(80))
            ), 'hex_codec'))

    totp_obj = googletotp.TOTP(base_32_secret.decode("utf-8"))
    qr_code = re.sub(r'=+$', '', totp_obj.provisioning_uri(user))
    key = str(base_32_secret, encoding="utf-8")
    DjangoGoogleAuthenticator2.objects.create(
        username=user,
        key=key,
    )

    return {
        "success":True,
        "data":qr_code
    }




from django.db import models

# Create your models here.

class DjangoGoogleAuthenticator2(models.Model):

    username = models.CharField(
        verbose_name="用户名",
        max_length=255,
    )

    key = models.CharField(
        verbose_name="秘钥",
        max_length=255,
        unique=True,
    )

    class Meta:
        db_table = 'google_authenticator2'



